@@ -16,33 +16,32 @@ from functools import reduce
 from typing import Any, Dict, List, Optional
 from uuid import UUID
 
+from dateutil import parser
 from sql import Column, Flavor, Table
 from sql.conditionals import Case
-from sql.operators import Equal, NotEqual
+from sql.operators import And, Equal, NotEqual
 
 from tikka.adapters.repository.categories import TABLE_NAME as CATEGORIES_TABLE_NAME
-from tikka.adapters.repository.sqlite3 import Sqlite3RepositoryInterface
+from tikka.adapters.repository.identities import TABLE_NAME as IDENTITIES_TABLE_NAME
 from tikka.adapters.repository.wallets import TABLE_NAME as WALLETS_TABLE_NAME
 from tikka.domains.entities.account import Account
 from tikka.interfaces.adapters.repository.accounts import AccountsRepositoryInterface
+from tikka.interfaces.adapters.repository.db_repository import DBRepositoryInterface
 
 TABLE_NAME = "accounts"
 
-# config sql with ? as param style
-Flavor.set(Flavor(paramstyle="qmark"))
 # create sql table wrapper
 sql_accounts_table = Table(TABLE_NAME)
 
 CATEGORIES_TABLE_NB_COLUMNS = 4
 CATEGORY_NAME_ALIAS_COLUMN = "category_name_alias"
 CATEGORY_ID_ALIAS_COLUMN = "category_id_alias"
+IDENTITY_NAME_ALIAS_COLUMN = "identity_name_alias"
 
 
-class Sqlite3AccountsRepository(
-    AccountsRepositoryInterface, Sqlite3RepositoryInterface
-):
+class DBAccountsRepository(AccountsRepositoryInterface, DBRepositoryInterface):
     """
-    Sqlite3AccountsRepository class
+    DBAccountsRepository class
     """
 
     def list(
@@ -60,17 +59,18 @@ class Sqlite3AccountsRepository(
             AccountsRepositoryInterface.COLUMN_NAME: sql_accounts_table.name,
             AccountsRepositoryInterface.COLUMN_CRYPTO_TYPE: sql_accounts_table.crypto_type,
             AccountsRepositoryInterface.COLUMN_BALANCE: sql_accounts_table.balance,
-            AccountsRepositoryInterface.COLUMN_IDENTITY_INDEX: sql_accounts_table.identity_index,
             AccountsRepositoryInterface.COLUMN_PATH: sql_accounts_table.path,
             AccountsRepositoryInterface.COLUMN_ROOT: sql_accounts_table.root,
             AccountsRepositoryInterface.COLUMN_FILE_IMPORT: sql_accounts_table.file_import,
             AccountsRepositoryInterface.COLUMN_CATEGORY_ID: sql_accounts_table.category_id,
+            AccountsRepositoryInterface.COLUMN_LEGACY_V1: sql_accounts_table.legacy_v1,
+            AccountsRepositoryInterface.COLUMN_TOTAL_TRANSFERS_COUNT: sql_accounts_table.total_transfers_count,
         }
 
         # if sort column...
         if sort_column is not None:
             # set sort column
-            sql_sort_colum = Column(sql_accounts_table, sort_column)
+            sql_sort_colum: Column = sql_columns[sort_column]
             # create select query wrapper with order by
             sql_select = sql_accounts_table.select(
                 order_by=sql_sort_colum.asc
@@ -87,9 +87,9 @@ class Sqlite3AccountsRepository(
             for key, value in filters.items():
                 if key == AccountsRepositoryInterface.COLUMN_ROOT:
                     if value is True:
-                        conditions.append(sql_accounts_table.root is None)
+                        conditions.append(Equal(sql_accounts_table.root, None))
                     elif value is False:
-                        conditions.append(sql_accounts_table.root is not None)
+                        conditions.append(NotEqual(sql_accounts_table.root, None))
                     elif isinstance(value, str):
                         conditions.append(
                             sql_accounts_table.root
@@ -100,10 +100,13 @@ class Sqlite3AccountsRepository(
 
         # conditions are added with and operator
         def and_(a, b):
-            return a & b
+            return And((a, b))
 
         if len(conditions) > 0:
-            sql_select.where = reduce(and_, conditions)
+            sql_select.where = reduce(and_, conditions)  # type: ignore
+
+        # config sql with ? as param style
+        Flavor.set(Flavor(paramstyle="qmark"))
 
         sql, args = tuple(sql_select)
         result_set = self.client.select(sql, args)
@@ -133,7 +136,8 @@ class Sqlite3AccountsRepository(
         # in order to do that, we join :
         # - the wallet table
 
-        # SELECT "e"."address", "a"."identity_index", "a"."balance", "a"."name", "a"."address", "a"."path", "a"."root", "a"."crypto_type",
+        # SELECT "e"."address", "a"."identity_index", "a"."balance", "a"."name", "a"."address", "a"."path", "a"."root",
+        # "a"."crypto_type",
         # CASE WHEN ("a"."root" IS NULL) THEN "b"."name" ELSE "d"."name" END AS "category_name_alias",
         # CASE WHEN ("a"."root" IS NULL) THEN "b"."id" ELSE "d"."id" END AS "category_id_alias"
         # FROM "accounts" AS "a"
@@ -166,38 +170,52 @@ class Sqlite3AccountsRepository(
         sql_wallets_join.condition = (
             sql_wallets_join.right.address == sql_accounts_table.address
         )
+        sql_identities_join = sql_wallets_join.join(
+            Table(IDENTITIES_TABLE_NAME), "LEFT"
+        )
+        sql_identities_join.condition = (
+            sql_identities_join.right.address == sql_accounts_table.address
+        )
+
+        # to sort by category name
+        category_name_alias = Case(
+            (Equal(sql_accounts_table.root, None), sql_categories_join.right.name),
+            else_=sql_categories_join2.right.name,
+        ).as_(CATEGORY_NAME_ALIAS_COLUMN)
+
+        # to filter by category id
+        category_id_alias = Case(
+            (Equal(sql_accounts_table.root, None), sql_categories_join.right.id),
+            else_=sql_categories_join2.right.id,
+        ).as_(CATEGORY_ID_ALIAS_COLUMN)
 
         # create select query wrapper
-        sql_select = sql_wallets_join.select(
+        sql_select = sql_identities_join.select(
             sql_wallets_join.right.address,
-            sql_accounts_table.identity_index,
+            sql_identities_join.right.index_,
+            sql_identities_join.right.name,
             sql_accounts_table.balance,
             sql_accounts_table.name,
             sql_accounts_table.address,
             sql_accounts_table.path,
             sql_accounts_table.root,
             sql_accounts_table.crypto_type,
-            Case(
-                (Equal(sql_accounts_table.root, None), sql_categories_join.right.name),
-                else_=sql_categories_join2.right.name,
-            ).as_(CATEGORY_NAME_ALIAS_COLUMN),
-            # add category_id column for category_id filter
-            Case(
-                (Equal(sql_accounts_table.root, None), sql_categories_join.right.id),
-                else_=sql_categories_join2.right.id,
-            ).as_(CATEGORY_ID_ALIAS_COLUMN),
+            sql_accounts_table.legacy_v1,
+            category_name_alias,
+            category_id_alias,
         )
 
         sql_sort_columns = [
             sql_wallets_join.right.address,
-            sql_accounts_table.identity_index,
-            sql_accounts_table.balance,
+            sql_identities_join.right.name,
             sql_accounts_table.name,
+            sql_accounts_table.balance,
             sql_accounts_table.address,
             sql_accounts_table.path,
             sql_accounts_table.root,
             sql_accounts_table.crypto_type,
-            sql_accounts_table.category_id.as_(CATEGORY_NAME_ALIAS_COLUMN),
+            sql_accounts_table.legacy_v1,
+            category_name_alias,
         ]
 
         # if sort column...
@@ -234,22 +252,24 @@ class Sqlite3AccountsRepository(
 
         # conditions are added with and operator
         def and_(a, b):
-            return a & b
+            return And((a, b))
 
         if len(conditions) > 0:
-            sql_select.where = reduce(and_, conditions)
+            sql_select.where = reduce(and_, conditions)  # type: ignore
 
         sql, args = tuple(sql_select)
         result_set = self.client.select(sql, args)
 
         list_ = []
         for row in result_set:
-            table_view_row = AccountsRepositoryInterface.TableViewRow(
-                *row[
-                    : len(
-                        AccountsRepositoryInterface.TableViewRow.__dataclass_fields__  # type: ignore # pylint: disable=no-member
-                    )
-                ],
+            table_view_row = get_table_view_row_from_row(
+                list(
+                    row[
+                        : len(
+                            AccountsRepositoryInterface.TableViewRow.__dataclass_fields__  # type: ignore
+                        )
+                    ]
+                )
             )
             list_.append(table_view_row)
         return list_
@@ -277,12 +297,19 @@ class Sqlite3AccountsRepository(
             **get_fields_from_account(account),
         )
 
-    def delete(self, account: Account) -> None:
+    def delete(self, address: str) -> None:
         __doc__ = (  # pylint: disable=redefined-builtin, unused-variable
             AccountsRepositoryInterface.delete.__doc__
         )
 
-        self.client.delete(TABLE_NAME, address=account.address)
+        self.client.delete(TABLE_NAME, address=address)
+
+    def delete_all(self) -> None:
+        __doc__ = (  # pylint: disable=redefined-builtin, unused-variable
+            AccountsRepositoryInterface.delete_all.__doc__
+        )
+
+        self.client.clear(TABLE_NAME)
 
     def count(self) -> int:
         __doc__ = (  # pylint: disable=redefined-builtin, unused-variable
@@ -294,6 +321,30 @@ class Sqlite3AccountsRepository(
 
         return row[0]
 
+    def total_balance(self) -> int:
+        __doc__ = (  # pylint: disable=redefined-builtin, unused-variable
+            AccountsRepositoryInterface.total_balance.__doc__
+        )
+        # fix error "integer overflow: SELECT sum(balance) FROM accounts"
+        row = self.client.select_one(
+            f"""WITH RECURSIVE partial_sums AS (
+        SELECT balance AS running_sum, rowid 
+        FROM {TABLE_NAME} 
+        WHERE rowid = 1
+        
+        UNION ALL
+        
+        SELECT a.balance + p.running_sum, a.rowid
+        FROM accounts a
+        JOIN partial_sums p ON a.rowid = p.rowid + 1
+    )
+    SELECT running_sum FROM partial_sums ORDER BY rowid DESC LIMIT 1"""
+        )
+        if row is None:
+            return 0
+
+        return row[0] or 0
+
 
 def get_fields_from_account(account: Account) -> dict:
     """
@@ -306,13 +357,32 @@ def get_fields_from_account(account: Account) -> dict:
     for (key, value) in account.__dict__.items():
         if key.startswith("_"):
             continue
-        if isinstance(value, UUID):
-            value = value.hex
-        fields[key] = value
+        elif isinstance(value, UUID):
+            fields[key] = value.hex
+        elif key == "balance" and value is not None:
+            # fix overflow with big numbers
+            fields[key] = str(value)
+        else:
+            fields[key] = value
 
     return fields
 
 
+def get_table_view_row_from_row(row: list) -> AccountsRepositoryInterface.TableViewRow:
+    """
+    Return AccountsRepositoryInterface.TableViewRow from DB result set row
+
+    :param row: Result set row
+    :return:
+    """
+    if row[3] is not None:
+        # convert to integer from str type
+        row[3] = int(row[3])
+    # cast legacy_v1 from int to bool
+    row[9] = bool(row[9])
+    return AccountsRepositoryInterface.TableViewRow(*row)
+
+
 def get_account_from_row(row: tuple) -> Account:
     """
     Return an Account instance from a result set row
@@ -323,10 +393,21 @@ def get_account_from_row(row: tuple) -> Account:
     values: List[Any] = []
     count = 0
     for value in row:
-        if count == 8 and value is not None:
-            values.append(UUID(hex=value))
-        elif count == 7:
+        if count == 3 and value is not None:
+            # convert to integer from str type
+            values.append(int(value))
+        elif count == 6:
             values.append(bool(value))
+        elif count == 7 and value is not None:
+            values.append(UUID(hex=value))
+        elif count == 8:
+            values.append(bool(value))
+        elif count == 10 and value is not None:
+            # python datetime does not handle timezone until 3.11
+            values.append(parser.parse(value))
+        elif count == 11 and value is not None:
+            # python datetime does not handle timezone until 3.11
+            values.append(parser.parse(value))
         else:
             values.append(value)
         count += 1
