@@ -15,21 +15,24 @@
 from typing import Any, Dict, List, Optional
 from uuid import UUID
 
-from duniterpy.key import SigningKey
-from substrateinterface import Keypair, KeypairType
-
 from tikka.domains.currencies import Currencies
-from tikka.domains.entities.account import Account
-from tikka.domains.entities.events import AccountEvent
+from tikka.domains.entities.account import Account, AccountCryptoType
+from tikka.domains.entities.constants import DERIVATION_SCAN_MAX_NUMBER
+from tikka.domains.entities.events import AccountEvent, LastTransferEvent
+from tikka.domains.entities.identity import Identity
 from tikka.domains.events import EventDispatcher
 from tikka.domains.passwords import Passwords
+from tikka.domains.transfers import Transfers
 from tikka.domains.wallets import Wallets
-from tikka.interfaces.adapters.network.accounts import NetworkAccountsInterface
+from tikka.interfaces.adapters.network.indexer.accounts import IndexerAccountsInterface
+from tikka.interfaces.adapters.network.node.accounts import NodeAccountsInterface
 from tikka.interfaces.adapters.repository.accounts import AccountsRepositoryInterface
 from tikka.interfaces.adapters.repository.file_wallets import (
     V1FileWalletsRepositoryInterface,
 )
 from tikka.libs.derivation import RE_TRANSPARENT_DERIVATION_PATH_PATTERN
+from tikka.libs.keypair import Keypair
+from tikka.libs.signing_key_v1 import SigningKey
 
 
 class Accounts:
@@ -40,9 +43,11 @@ class Accounts:
     def __init__(
         self,
         repository: AccountsRepositoryInterface,
-        network: NetworkAccountsInterface,
+        network_node: NodeAccountsInterface,
+        network_indexer: IndexerAccountsInterface,
         passwords: Passwords,
         wallets: Wallets,
+        transfers: Transfers,
         file_wallets_repository: V1FileWalletsRepositoryInterface,
         currencies: Currencies,
         event_dispatcher: EventDispatcher,
@@ -51,21 +56,42 @@ class Accounts:
         Init Accounts domain
 
         :param repository: AccountsRepositoryInterface instance
-        :param network: NetworkAccountsInterface instance
+        :param network_node: NodeAccountsInterface instance
+        :param network_indexer: IndexerAccountsInterface instance
         :param passwords: Passwords domain instance
         :param wallets: Wallets domain instance
+        :param transfers: Transfers domain instance
         :param file_wallets_repository: FileWalletsRepository adapter instance
         :param currencies: Currencies domain instance
         :param event_dispatcher: EventDispatcher instance
         """
         self.repository = repository
-        self.network = network
+        self.network_node = network_node
+        self.network_indexer = network_indexer
         self.passwords = passwords
         self.wallets = wallets
+        self.transfers = transfers
         self.file_wallets_repository = file_wallets_repository
         self.currencies = currencies
         self.event_dispatcher = event_dispatcher
 
+        # subscribe to events
+        self.event_dispatcher.add_event_listener(
+            LastTransferEvent.EVENT_TYPE_LAST_TRANSFER_CHANGED,
+            self.on_last_transfer_changed,
+        )
+
+    @staticmethod
+    def get_instance(address: str, name: Optional[str] = None) -> Account:
+        """
+        Return an Account instance from arguments
+
+        :param address: Account address
+        :param name: Optional account name
+        :return:
+        """
+        return Account(address=address, name=name)
+
     def get_list(
         self,
         filters: Optional[Dict[str, Any]] = None,
@@ -145,11 +171,7 @@ class Accounts:
         :param account: Account instance to delete
         :return:
         """
-        self.repository.delete(account)
-        self.wallets.delete(account.address)
-        # if password exists for this root account...
-        if self.passwords.exists(account.address):
-            self.passwords.delete(account.address)
+        self.repository.delete(account.address)
 
         # dispatch event
         event = AccountEvent(
@@ -158,6 +180,14 @@ class Accounts:
         )
         self.event_dispatcher.dispatch_event(event)
 
+    def delete_all(self) -> None:
+        """
+        Delete all accounts in repository
+
+        :return:
+        """
+        self.repository.delete_all()
+
     def count(self) -> int:
         """
         Return total number of accounts
@@ -166,6 +196,14 @@ class Accounts:
         """
         return self.repository.count()
 
+    def get_total_balance(self) -> int:
+        """
+        Return total balance of all accounts
+
+        :return:
+        """
+        return self.repository.total_balance()
+
     def unlock(self, account: Account, wallet_password: str) -> bool:
         """
         Unlock wallet of account if wallet_password is OK
@@ -178,12 +216,7 @@ class Accounts:
         if result is False:
             return False
 
-        # dispatch event
-        event = AccountEvent(
-            AccountEvent.EVENT_TYPE_UPDATE,
-            account,
-        )
-        self.event_dispatcher.dispatch_event(event)
+        self.update(account)
         return True
 
     def lock(self, account: Account):
@@ -214,16 +247,88 @@ class Accounts:
             AccountEvent(AccountEvent.EVENT_TYPE_UPDATE, account)
         )
 
-    def fetch_balance_from_network(self, account: Account) -> None:
+    def network_update_balance(self, account: Account) -> Account:
         """
-        Fetch account balance from current EntryPoint connection
+        Update account balance from network
 
         :param account: Account instance
         :return:
         """
-        balance = self.network.get_balance(account.address)
-        if balance is not None:
-            account.balance = balance
+        account.balance = self.network_node.get_balance(account.address)
+        self.repository.update(account)
+
+        return account
+
+    def network_update_account(self, account: Account) -> Account:
+        """
+        Update
+        - balance
+        - total_transfers_count
+        from network
+
+        :param account: Account instance
+        :return:
+        """
+        account.balance = self.network_node.get_balance(account.address)
+        self.repository.update(account)
+
+        return account
+
+    def network_update_balances(self, accounts: List[Account]) -> None:
+        """
+        Update balances of account list from network
+
+        :param accounts: Account list
+        :return:
+        """
+        balances = self.network_node.get_balances(
+            [account.address for account in accounts]
+        )
+        for account in accounts:
+            account.balance = balances[account.address]
+            self.repository.update(account)
+
+    def network_get_balance(self, address: str) -> Optional[int]:
+        """
+        Return account balance from network
+
+        :param address: Account address
+        :return:
+        """
+        return self.network_node.get_balance(address)
+
+    def network_get_balances(self, addresses: List[str]) -> Dict[str, Optional[int]]:
+        """
+        Return balances of address list from network
+
+        :param addresses: Account address list
+        :return:
+        """
+        return self.network_node.get_balances(addresses)
+
+    def network_get_unclaimed_ud_balance(self, identity: Identity) -> int:
+        """
+        Return the unclaimed UD balance of identity from network
+
+        :param identity: Identity instance
+        :return:
+        """
+        balance = self.network_node.get_unclaimed_ud_balance(identity.first_eligible_ud)
+
+        return balance
+
+    def network_update_total_transfers_count(self, account: Account) -> Account:
+        """
+        Update account total_transfers_count from network
+
+        :param account: Account to update
+        :return:
+        """
+        account.total_transfers_count = (
+            self.transfers.network_fetch_total_count_for_address(account.address)
+        )
+        self.repository.update(account)
+        return account
 
     def list_by_category_id(self, category_id: Optional[UUID]) -> List[Account]:
         """
@@ -250,109 +355,146 @@ class Accounts:
         :return:
         """
         filters = {AccountsRepositoryInterface.COLUMN_ROOT: address}
-        return self.repository.list(filters=filters)
+        # sort ascending by derivation path numbers
+        return self.repository.list(
+            filters=filters,
+            sort_column=AccountsRepositoryInterface.COLUMN_PATH,
+            sort_order=AccountsRepositoryInterface.SORT_ORDER_ASCENDING,
+        )
 
     def create_new_account(
         self,
         mnemonic: str,
         language_code: str,
         derivation: str,
+        crypto_type: AccountCryptoType,
         name: str,
         password: str,
-    ) -> None:
+        add_event: bool = True,
+    ) -> Account:
         """
-        Create a root account (read-only if derivation not empty) with mnemonic, and a transparent derivation account as per RFC 0019,
+        Create a root account (read-only if derivation not empty) with mnemonic, if it does not already exist,
+        and create a derived account if derivation is not "" as per RFC 0019.
+        Return root account if derivation == "", derived account otherwise.
 
         :param mnemonic: Mnemonic phrase
         :param language_code: Mnemonic language code
         :param derivation: Derivation path
+        :param crypto_type: Key type as AccountCryptoType instance
         :param name: Account name
         :param password: Wallet password
+        :param add_event: Trigger event if True
         :return:
         """
         # get root keypair
         root_keypair = Keypair.create_from_mnemonic(
             mnemonic=mnemonic,
             language_code=language_code,
-            crypto_type=KeypairType.SR25519,
+            crypto_type=crypto_type,
             ss58_format=self.currencies.get_current().ss58_format,
         )
-        root_account = self.get_by_address(root_keypair.ss58_address)
-        if root_account is None:
+        existing_account = self.get_by_address(root_keypair.ss58_address)
+        if existing_account is None:
             # store new password for new root account
-            self.passwords.new(root_keypair, password)
             if derivation == "":
-                self.create_new_root_account(mnemonic, language_code, name, password)
+                self.create_new_root_account(
+                    mnemonic, language_code, crypto_type, name, password
+                )
             else:
                 # read-only root account
-                self.create_new_root_account(mnemonic, language_code, None, None)
+                self.create_new_root_account(
+                    mnemonic, language_code, crypto_type, None, None, False
+                )
+            self.passwords.new(root_keypair, password)
             root_account = self.get_by_address(root_keypair.ss58_address)
         else:
             # get password of root account
             clear_password = self.passwords.get_clear_password(root_keypair)
             if clear_password is not None:
                 password = clear_password
+            root_account = existing_account
 
-        if derivation == "" or root_account is None:
-            return
+        if derivation == "":
+            return root_account  # type: ignore
 
         # create keypair from mnemonic and path
         keypair = Keypair.create_from_uri(
             suri=mnemonic + derivation,
             language_code=language_code,
-            crypto_type=KeypairType.SR25519,
+            crypto_type=crypto_type,
             ss58_format=self.currencies.get_current().ss58_format,
         )
 
+        # create and store Account instance
+        account = Account(
+            keypair.ss58_address,
+            crypto_type=crypto_type,
+            legacy_v1=False,
+            name=None if name == "" else name,
+            root=root_account.address,  # type: ignore
+            path=derivation,
+        )
+
+        if add_event:
+            self.add(account)
+        else:
+            self.repository.add(account)
+
         # create and store Wallet instance
         wallet = self.wallets.create(keypair, password)
         if not self.wallets.exists(keypair.ss58_address):
             self.wallets.add(wallet)
             self.wallets.unlock(keypair.ss58_address, password)
 
-        # create and store Account instance
-        account = Account(
-            keypair.ss58_address,
-            crypto_type=KeypairType.SR25519,
-            name=None if name == "" else name,
-            root=root_account.address,
-            path=derivation,
-        )
-        self.add(account)
+        return account
 
     def create_new_root_account(
         self,
         mnemonic: str,
         language_code: str,
+        crypto_type: AccountCryptoType,
         name: Optional[str],
         password: Optional[str],
-    ) -> None:
+        add_event: bool = True,
+    ) -> Account:
         """
-        Create a read-only root account with mnemonic
+        Create and return a root account from mnemonic
 
         :param mnemonic: Mnemonic phrase
         :param language_code: Mnemonic language code
+        :param crypto_type: Key type as AccountCryptoType instance
         :param name: Optional account name
         :param password: Optional Wallet password, if None no wallet will be created
+        :param add_event: Optional, Default True to trigger add event
         :return:
         """
+
         # get root keypair
         root_keypair = Keypair.create_from_mnemonic(
             mnemonic=mnemonic,
             language_code=language_code,
-            crypto_type=KeypairType.SR25519,
+            crypto_type=crypto_type,
             ss58_format=self.currencies.get_current().ss58_format,
         )
+
         root_account = Account(
-            root_keypair.ss58_address, crypto_type=KeypairType.SR25519, name=name
+            root_keypair.ss58_address, crypto_type=crypto_type, name=name
         )
+
+        if add_event:
+            self.add(root_account)
+        else:
+            # add root account in repository without event
+            self.repository.add(root_account)
+
         if password is not None:
             # create and store Wallet instance
             wallet = self.wallets.new(root_keypair, password)
             self.wallets.unlock(wallet.address, password)
+            # store new password for new V1 root account
+            self.passwords.new(root_keypair, password)
 
-        # add root account in repository without event
-        self.add(root_account)
+        return root_account
 
     def create_new_root_account_v1_from_credentials(
         self, secret_id: str, password_id: str, name: str, password: str
@@ -385,27 +527,29 @@ class Accounts:
         keypair = Keypair.create_from_seed(
             seed_hex=seed_hex,
             ss58_format=self.currencies.get_current().ss58_format,
-            crypto_type=KeypairType.ED25519,
+            crypto_type=AccountCryptoType.ED25519,
         )
         address = keypair.ss58_address
-        if self.get_by_address(address) is not None:
-            return None
+        account = self.get_by_address(address)
+        if account is not None:
+            return account
 
-        # store new password for new V1 root account
-        self.passwords.new(keypair, password)
+        # create and store Account instance
+        account = Account(
+            address,
+            crypto_type=AccountCryptoType.ED25519,
+            legacy_v1=True,
+            name=None if name == "" else name,
+        )
+        self.add(account)
 
         # create and store Wallet instance
         wallet = self.wallets.create(keypair, password)
         self.wallets.add(wallet)
         self.wallets.unlock(wallet.address, password)
 
-        # create and store Account instance
-        account = Account(
-            address,
-            crypto_type=KeypairType.ED25519,
-            name=None if name == "" else name,
-        )
-        self.add(account)
+        # store new password for new V1 root account
+        self.passwords.new(keypair, password)
 
         return account
 
@@ -420,26 +564,25 @@ class Accounts:
         available_derivation_list = []
 
         # get max existing transparent derivation number
-        derivation_number = 0
-        member_derivation_exists = False
+        derivation_numbers: List[int] = []
+        max_derivation_number = 0
         for account in derived_accounts:
             if account.path is None:
                 continue
             match = RE_TRANSPARENT_DERIVATION_PATH_PATTERN.fullmatch(account.path)
             if match:
                 account_derivation_number = int(match.group(1))
-                if account_derivation_number > 0 and account_derivation_number % 2 == 0:
-                    derivation_number = max(
-                        derivation_number, account_derivation_number
-                    )
-                if account_derivation_number == 0:
-                    member_derivation_exists = True
+                derivation_numbers.append(account_derivation_number)
+                max_derivation_number = max(
+                    max_derivation_number, account_derivation_number
+                )
 
-        # add next available transparent derivation path
-        available_derivation_list.append(f"//{derivation_number + 2}")
-
-        if not member_derivation_exists:
-            available_derivation_list.append("//0")
+        for derivation_number in range(
+            0, max(max_derivation_number, DERIVATION_SCAN_MAX_NUMBER) + 1, 1
+        ):
+            if derivation_number not in derivation_numbers:
+                # add next available transparent derivation path
+                available_derivation_list.append(f"//{derivation_number}")
 
         return available_derivation_list
 
@@ -482,7 +625,28 @@ class Accounts:
                 keypair = Keypair.create_from_uri(
                     suri=suri,
                     language_code=language_code,
-                    crypto_type=KeypairType.SR25519,
-                    ss58_format=self.currencies.get_current().ss58_format,
+                    crypto_type=root_keypair.crypto_type,
+                    ss58_format=root_keypair.ss58_format,
                 )
                 self.wallets.update(self.wallets.create(keypair, password))
+
+    def on_last_transfer_changed(self, event: LastTransferEvent):
+        """
+        Update account with last transfer timestamp got from network
+
+        :param event: LastTransferEvent instance
+        :return:
+        """
+        account = self.get_by_address(event.address)
+        if account is not None:
+            account.last_transfer_timestamp = event.transfer.timestamp
+            self.repository.update(account)
+
+    def network_is_legacy_v1(self, address: str) -> bool:
+        """
+        Return True if account is a legacy v1 account (present in genesis)
+
+        :param address: ss58 account address
+        :return:
+        """
+        return self.network_indexer.is_legacy_v1(address)
