@@ -23,6 +23,26 @@ class Category:
     name: str
     expanded: bool = True
     parent_id: Optional[UUID] = None
+    _balance: int = 0
+
+    @property
+    def balance(self):
+        """
+        Return hidden field _balance
+
+        :return:
+        """
+        return self._balance
+
+    @balance.setter
+    def balance(self, value):
+        """
+        Set value of hidden _balance
+
+        :param value: Balance value
+        :return:
+        """
+        self._balance = value
 
     def __eq__(self, other):
         """
