@@ -15,17 +15,20 @@
 from dataclasses import dataclass
 from typing import Optional
 
-TABLE_NAME = "nodes"
+from tikka.interfaces.entities.Server import ServerInterface
 
 
 @dataclass
-class Node:
+class Node(ServerInterface):
 
     url: str
     peer_id: Optional[str] = None
     block: Optional[int] = None
     software: Optional[str] = None
     software_version: Optional[str] = None
+    session_keys: Optional[str] = None
+    epoch_index: Optional[int] = None
+    unsafe_api_exposed: bool = False
 
     def __str__(self):
         """
