@@ -16,7 +16,7 @@ from dataclasses import dataclass
 from pathlib import Path
 from typing import Optional
 
-from duniterpy.key import SigningKey
+from tikka.libs.signing_key_v1 import SigningKey
 
 WALLET_TYPES = {
     "PUBSEC": "PubSec: a file with your secret and public key",
