@@ -12,23 +12,25 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
+import json
+import logging
 import random
+from json import JSONDecodeError
 from typing import List, Optional
+from urllib import request
 
-from tikka.adapters.network.nodes import NetworkNodes
-from tikka.adapters.network.rpc.connection import RPCConnection
+from tikka.adapters.network.node.node import NetworkNode
 from tikka.domains.config import Config
-from tikka.domains.connections import Connections
 from tikka.domains.currencies import Currencies
-from tikka.domains.entities.events import ConnectionsEvent, NodesEvent
+from tikka.domains.entities.constants import (
+    NODES_CURRENT_ENTRY_POINT_URL_PREFERENCES_KEY,
+)
+from tikka.domains.entities.events import NodesEvent
 from tikka.domains.entities.node import Node
 from tikka.domains.events import EventDispatcher
-from tikka.interfaces.adapters.network.nodes import NetworkNodesInterface
+from tikka.domains.preferences import Preferences
+from tikka.interfaces.adapters.network.node.node import NetworkNodeInterface
 from tikka.interfaces.adapters.repository.nodes import NodesRepositoryInterface
-from tikka.interfaces.adapters.repository.preferences import (
-    CURRENT_ENTRY_POINT_URL,
-    PreferencesRepositoryInterface,
-)
 
 
 class Nodes:
@@ -36,21 +38,14 @@ class Nodes:
     Nodes domain class
     """
 
-    DEFAULT_LIST_OFFSET = 0
-    DEFAULT_LIST_LIMIT = 1000
-
-    PROPERTY_URL = NodesRepositoryInterface.COLUMN_URL
-    PROPERTY_PEER_ID = NodesRepositoryInterface.COLUMN_PEER_ID
-    PROPERTY_BLOCK = NodesRepositoryInterface.COLUMN_BLOCK
-    PROPERTY_SOFTWARE = NodesRepositoryInterface.COLUMN_SOFTWARE
-    PROPERTY_SOFTWARE_VERSION = NodesRepositoryInterface.COLUMN_SOFTWARE_VERSION
+    CONFIG_NODES_ENDPOINTS_KEYWORD = "rpc"
+    CONFIG_ONLINE_ENDPOINTS_KEYWORD = "online"
 
     def __init__(
         self,
         repository: NodesRepositoryInterface,
-        preferences_repository: PreferencesRepositoryInterface,
-        connections: Connections,
-        network: NetworkNodesInterface,
+        preferences: Preferences,
+        network_node: NetworkNodeInterface,
         config: Config,
         currencies: Currencies,
         event_dispatcher: EventDispatcher,
@@ -59,29 +54,24 @@ class Nodes:
         Init Nodes domain instance
 
         :param repository: EntryPointsRepositoryInterface instance
-        :param preferences_repository: PreferencesRepositoryInterface instance
-        :param connections: Connections instance
-        :param network: Network adapter instance for handling nodes
+        :param preferences: Preferences domain instance
+        :param network_node: NetworkNodeInterface adapter instance for handling nodes
         :param config: Config instance
         :param currencies: Currencies instance
         :param event_dispatcher: EventDispatcher instance
         """
         self.repository = repository
-        self.preferences_repository = preferences_repository
-        self.connections = connections
-        self.network = network
+        self.preferences = preferences
+        self.network_node = network_node
         self.config = config
         self.currencies = currencies
         self.event_dispatcher = event_dispatcher
-        self.current_url = self.currencies.get_entry_point_urls()[0]
+        self._current_url = self.currencies.get_entry_point_urls()[
+            self.CONFIG_NODES_ENDPOINTS_KEYWORD
+        ][0]
 
         self.init_repository()
 
-        # events
-        self.event_dispatcher.add_event_listener(
-            ConnectionsEvent.EVENT_TYPE_CONNECTED, self._on_connections_connected
-        )
-
     def init_repository(self):
         """
         Init repository with default entry points from config
@@ -89,24 +79,56 @@ class Nodes:
         :return:
         """
         repository_urls = self.repository.get_urls()
+        currency_endpoints = self.currencies.get_entry_point_urls()
 
         # init repository with current currency entry point urls
-        for url in self.currencies.get_entry_point_urls():
+        for url in currency_endpoints[self.CONFIG_NODES_ENDPOINTS_KEYWORD]:
             if url not in repository_urls:
                 self.repository.add(Node(url))
 
-        self.current_url = self.repository.list(0, 1)[0].url
+        self._current_url = self.repository.list(0, 1)[0].url
 
-        current_url_in_preferences = self.preferences_repository.get(
-            CURRENT_ENTRY_POINT_URL
+        current_url_in_preferences = self.preferences.get(
+            NODES_CURRENT_ENTRY_POINT_URL_PREFERENCES_KEY
         )
         if (
             current_url_in_preferences is None
             or current_url_in_preferences not in self.repository.get_urls()
         ):
-            self.preferences_repository.set(CURRENT_ENTRY_POINT_URL, self.current_url)
+            self.preferences.set(
+                NODES_CURRENT_ENTRY_POINT_URL_PREFERENCES_KEY, self.get_current_url()
+            )
         else:
-            self.current_url = current_url_in_preferences
+            self._current_url = current_url_in_preferences
+
+    def network_fetch_endpoints(self):
+        """
+        Add Indexers in repository from online list of endpoints
+
+        :return:
+        """
+        currency_endpoints = self.currencies.get_entry_point_urls()
+
+        # fetch endpoints from online json file
+        if currency_endpoints[self.CONFIG_ONLINE_ENDPOINTS_KEYWORD]:
+            try:
+                with request.urlopen(
+                    currency_endpoints[self.CONFIG_ONLINE_ENDPOINTS_KEYWORD]
+                ) as file:
+                    online_endpoints = json.load(file)
+            except JSONDecodeError as exception:
+                logging.warning(
+                    "error loading endpoints from %s",
+                    currency_endpoints[self.CONFIG_ONLINE_ENDPOINTS_KEYWORD],
+                )
+                logging.exception(exception)
+                online_endpoints = None
+
+            if online_endpoints is not None:
+                repository_urls = self.repository.get_urls()
+                for url in online_endpoints[self.CONFIG_NODES_ENDPOINTS_KEYWORD]:
+                    if url not in repository_urls:
+                        self.repository.add(Node(url))
 
     def network_fetch_current_node(self) -> None:
         """
@@ -114,34 +136,38 @@ class Nodes:
 
         :return:
         """
-        node = self.network.get()
-        if node is None:
+        current_node = self.repository.get(self.get_current_url())
+        network_node = self.network_node.get()
+        if network_node is None:
             return None
 
-        self.repository.update(node)
+        if current_node is not None:
+            # update only changing properties
+            current_node.block = network_node.block
+            current_node.peer_id = network_node.peer_id
+            current_node.software = network_node.software
+            current_node.software_version = network_node.software_version
+            current_node.epoch_index = network_node.epoch_index
+            current_node.unsafe_api_exposed = network_node.unsafe_api_exposed
+
+            self.repository.update(current_node)
 
         return None
 
     @staticmethod
-    def network_test_and_get_node(url: str) -> Optional[Node]:
+    def network_get_node_adapter(url: str) -> Optional[NetworkNode]:
         """
-        Try to open connection on url and return node if successful
-
-        Then close connection
+        Try to open connection on url and return connected NetworkNode instance if successful
 
         :param url: Entry point url
         :return:
         """
-        node = None
+        network_node = NetworkNode()
+        network_node.connection.connect(Node(url))
+        if network_node.connection.is_connected():
+            return network_node
 
-        connections = Connections(RPCConnection(), EventDispatcher())
-        connections.connect(Node(url))
-        if connections.is_connected():
-            network_nodes = NetworkNodes(connections)
-            node = network_nodes.get()
-        connections.disconnect()
-
-        return node
+        return None
 
     def set_current_url_randomly(self):
         """
@@ -152,9 +178,12 @@ class Nodes:
         indices = list(range(1, self.count()))
         random.shuffle(indices)
         for index in indices:
-            self.set_current_url(self.list()[index].url)
-            if self.connections.is_connected():
-                break
+            url = self.list()[index].url
+            # never choose localhost randomly...
+            if "localhost" not in url:
+                self.set_current_url(self.list()[index].url)
+                if self.network_node.connection.is_connected():
+                    break
 
     def add(self, node: Node) -> None:
         """
@@ -215,26 +244,36 @@ class Nodes:
         :return:
         """
         # do not delete default entry points from config
-        if url in self.currencies.get_entry_point_urls():
+        if url in self.currencies.get_entry_point_urls()["rpc"]:
             return
 
         self.repository.delete(url)
         # switch current entry point to first in list
         self.set_current_url(self.repository.list(0, 1)[0].url)
         # set new entry point in preferences
-        self.preferences_repository.set(CURRENT_ENTRY_POINT_URL, self.get_current_url())
+        self.preferences.set(
+            NODES_CURRENT_ENTRY_POINT_URL_PREFERENCES_KEY, self.get_current_url()
+        )
 
         self.event_dispatcher.dispatch_event(
             NodesEvent(NodesEvent.EVENT_TYPE_LIST_CHANGED)
         )
 
+    def delete_all(self) -> None:
+        """
+        Delete all nodes in repository
+
+        :return:
+        """
+        self.repository.delete_all()
+
     def get_current_url(self) -> str:
         """
         Return current entry point url
 
         :return:
         """
-        return self.current_url
+        return self._current_url
 
     def set_current_url(self, url: str) -> None:
         """
@@ -242,23 +281,14 @@ class Nodes:
 
         :return:
         """
-        self.current_url = url
+        self._current_url = url
         # update preference
-        self.preferences_repository.set(
-            CURRENT_ENTRY_POINT_URL,
-            self.current_url,
+        self.preferences.set(
+            NODES_CURRENT_ENTRY_POINT_URL_PREFERENCES_KEY,
+            self._current_url,
         )
         # switch current connection
-        self.connections.disconnect()
-        node = self.get(self.current_url)
+        self.network_node.connection.disconnect()
+        node = self.get(self._current_url)
         if node is not None:
-            self.connections.connect(node)
-
-    def _on_connections_connected(self, _: ConnectionsEvent):
-        """
-        Triggered when the connection is established
-
-        :param _:
-        :return:
-        """
-        self.network_fetch_current_node()
+            self.network_node.connection.connect(node)
