@@ -13,13 +13,13 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 import hashlib
-import logging
 import random
 import re
 import string
 
 from mnemonic import Mnemonic
 
+from tikka import __version__
 from tikka.domains.entities.constants import MNEMONIC_LANGUAGES, WALLETS_PASSWORD_LENGTH
 
 
@@ -31,7 +31,7 @@ def generate_alphabetic(size: int = 5) -> str:
     :return:
     """
     # fixme: remove this for production release
-    if logging.getLogger().getEffectiveLevel() == logging.DEBUG:
+    if int(__version__.split(".")[0]) < 1:
         return "A" * WALLETS_PASSWORD_LENGTH
 
     return "".join(random.choice(string.ascii_letters).upper() for _ in range(size))
