@@ -12,11 +12,12 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
+import logging
 import sys
 from typing import Optional
 
-from PyQt5.QtCore import QModelIndex, QMutex, QPoint, QRect
-from PyQt5.QtGui import QPainter, QPixmap
+from PyQt5.QtCore import QModelIndex, QMutex, QPoint, QRect, Qt
+from PyQt5.QtGui import QBrush, QColor, QFontMetrics, QPainter, QPixmap
 from PyQt5.QtWidgets import (
     QApplication,
     QMainWindow,
@@ -28,9 +29,23 @@ from PyQt5.QtWidgets import (
 from tikka.domains.application import Application
 from tikka.domains.entities.account import Account
 from tikka.domains.entities.category import Category
-from tikka.domains.entities.constants import DATA_PATH
-from tikka.domains.entities.events import AccountEvent, CategoryEvent, CurrencyEvent
-from tikka.slots.pyqt.models.account_tree import AccountTreeModel, MemberStatusRole
+from tikka.domains.entities.constants import AMOUNT_UNIT_KEY, DATA_PATH
+from tikka.domains.entities.events import (
+    AccountEvent,
+    CategoryEvent,
+    CurrencyEvent,
+    UnitEvent,
+)
+from tikka.slots.pyqt.entities.constants import (
+    NUMERIC_DISPLAY_COLOR_BLUE,
+    SELECTED_UNIT_PREFERENCES_KEY,
+)
+from tikka.slots.pyqt.entities.worker import AsyncQWorker
+from tikka.slots.pyqt.models.account_tree import (
+    AccountTreeModel,
+    BalanceStyledRole,
+    WalletLockStatusRole,
+)
 from tikka.slots.pyqt.resources.gui.widgets.account_tree_rc import Ui_AccountTreeWidget
 from tikka.slots.pyqt.widgets.account_menu import AccountPopupMenu
 from tikka.slots.pyqt.widgets.category_menu import CategoryPopupMenu
@@ -52,16 +67,94 @@ class AccountIconsItemDelegate(QStyledItemDelegate):
         :param index: QModelIndex instance
         :return:
         """
+        if not index.isValid():
+            return
         super().paint(painter, option, index)
-        member_pixmap: QPixmap = index.data(MemberStatusRole)
-        if isinstance(member_pixmap, QPixmap):
-            member_rect = QRect(
-                option.rect.x() - member_pixmap.rect().width() - 2,
-                option.rect.y(),
-                member_pixmap.rect().width(),
-                member_pixmap.rect().height(),
+
+        wallet_lock_status_pixmap = index.data(WalletLockStatusRole)
+        if isinstance(wallet_lock_status_pixmap, QPixmap):
+            icon_x = option.rect.x() - wallet_lock_status_pixmap.rect().width() - 2
+            icon_y = (
+                option.rect.y()
+                + (option.rect.height() - wallet_lock_status_pixmap.height()) // 2
             )
-            painter.drawPixmap(member_rect, member_pixmap)
+            icon_rect = QRect(
+                icon_x,
+                icon_y,
+                wallet_lock_status_pixmap.width(),
+                wallet_lock_status_pixmap.height(),
+            )
+            painter.drawPixmap(icon_rect, wallet_lock_status_pixmap)
+
+
+class StyledBalanceItemDelegate(QStyledItemDelegate):
+    """
+    Class used to display balance with a custom style
+    """
+
+    def paint(
+        self, painter: QPainter, option: QStyleOptionViewItem, index: QModelIndex
+    ) -> None:
+        """
+        Return balance with a custom style
+
+        :param painter: QPainter instance
+        :param option: QStyleOptionViewItem instance
+        :param index: QModelIndex instance
+        :return:
+        """
+        if not index.isValid():
+            return
+        super().paint(painter, option, index)
+
+        # draw balance with style
+        balance_styled: str = index.data(BalanceStyledRole)
+        if balance_styled:
+            painter.save()
+
+            # Enable anti-aliasing for smoother rendering
+            painter.setRenderHint(QPainter.Antialiasing)
+
+            # Set text color and font
+            painter.setPen(Qt.white)
+            painter.setFont(option.font)
+
+            # Get text size using boundingRect
+            font_metrics = QFontMetrics(option.font)
+            text_rect = font_metrics.boundingRect(balance_styled)
+
+            # Define padding and right margin
+            padding_x, padding_y = 8, 4
+            right_margin = 10  # Adjust this value for more space on the right
+
+            # Adjust background rectangle size to include the right margin
+            bg_width = text_rect.width() + 2 * padding_x + right_margin
+            bg_height = text_rect.height() + 2 * padding_y
+
+            # Position the rectangle to be right-aligned
+            rect_x = option.rect.right() - bg_width - 5  # 5px margin from the right
+            rect_y = option.rect.center().y() - bg_height // 2  # Centered vertically
+
+            # Draw rounded rectangle
+            painter.setBrush(
+                QBrush(QColor(NUMERIC_DISPLAY_COLOR_BLUE))
+            )  # Blue background
+            # painter.setPen(Qt.NoPen)
+            painter.drawRoundedRect(QRect(rect_x, rect_y, bg_width, bg_height), 10, 10)
+
+            # Draw text inside the rectangle (horizontally centered with right margin)
+            painter.drawText(
+                QRect(
+                    rect_x + padding_x,
+                    rect_y + padding_y,
+                    bg_width - padding_x - right_margin,
+                    bg_height - 2 * padding_y,
+                ),
+                Qt.AlignCenter,
+                balance_styled,
+            )
+
+            painter.restore()
 
 
 class AccountTreeWidget(QWidget, Ui_AccountTreeWidget):
@@ -86,20 +179,41 @@ class AccountTreeWidget(QWidget, Ui_AccountTreeWidget):
         self.setupUi(self)
 
         self.application = application
+        self._ = self.application.translator.gettext
         self.mutex = mutex
-        self.account_tree_model = AccountTreeModel(self.application)
+        self.error_message: Optional[str] = None
+
+        self.account_tree_model = AccountTreeModel(self.application, self.locale())
         self.treeView.setModel(self.account_tree_model)
         self.treeView.setHeaderHidden(True)
         self.treeView.expandAll()
         self.treeView.resizeColumnToContents(0)
+
         # set style sheet
         self.treeView.setStyleSheet("QTreeView::item {" "padding: 5px;" "}")
-        self.treeView.setItemDelegateForColumn(0, AccountIconsItemDelegate())
+        self.account_icons_item_delegate = AccountIconsItemDelegate()
+        self.styled_balance_item_delegate = StyledBalanceItemDelegate()
+        self.treeView.setItemDelegateForColumn(0, self.account_icons_item_delegate)
+        self.treeView.setItemDelegateForColumn(3, self.styled_balance_item_delegate)
+
         # init expand/collapse state of categories
         self.recursive_expand_or_collapse_categories(QModelIndex())
+        self.treeView.resizeColumnToContents(1)
+
+        ##############################
+        # ASYNC METHODS
+        ##############################
+        # Create a QWorker object
+        self.fetch_from_network_async_qworker = AsyncQWorker(
+            self.fetch_from_network, self.mutex
+        )
+        self.fetch_from_network_async_qworker.finished.connect(
+            self._on_finished_fetch_from_network
+        )
 
         # events
         self.addCategoryButton.clicked.connect(self.on_add_category_button_clicked)
+        self.refreshButton.clicked.connect(self.on_refresh_button_clicked)
         self.treeView.customContextMenuRequested.connect(self.on_context_menu)
         self.treeView.expanded.connect(self.on_expanded)
         self.treeView.collapsed.connect(self.on_collapsed)
@@ -112,6 +226,9 @@ class AccountTreeWidget(QWidget, Ui_AccountTreeWidget):
         self.application.event_dispatcher.add_event_listener(
             CategoryEvent.EVENT_TYPE_DELETE, self.on_category_delete_event
         )
+        self.application.event_dispatcher.add_event_listener(
+            UnitEvent.EVENT_TYPE_CHANGED, self.on_unit_event
+        )
         self.application.event_dispatcher.add_event_listener(
             CurrencyEvent.EVENT_TYPE_CHANGED, self.on_currency_event
         )
@@ -124,6 +241,8 @@ class AccountTreeWidget(QWidget, Ui_AccountTreeWidget):
         self.application.event_dispatcher.add_event_listener(
             AccountEvent.EVENT_TYPE_UPDATE, self.on_update_account_event
         )
+        # display total balance
+        self.update_ui()
 
     def rename_category(self):
         """
@@ -223,6 +342,29 @@ class AccountTreeWidget(QWidget, Ui_AccountTreeWidget):
         self.treeView.resizeColumnToContents(0)
         self.recursive_expand_or_collapse_categories(QModelIndex())
 
+    def update_ui(self) -> None:
+        """
+        Update UI data
+
+        :return:
+        """
+        if self.error_message is None:
+            self.errorLabel.setText(self._(self.error_message))
+        unit_preference = self.application.repository.preferences.get(
+            SELECTED_UNIT_PREFERENCES_KEY
+        )
+        if unit_preference is not None:
+            amount = self.application.amounts.get_amount(unit_preference)
+        else:
+            amount = self.application.amounts.get_amount(AMOUNT_UNIT_KEY)
+        total_balance = self.application.accounts.get_total_balance()
+        self.totalBalanceValueLabel.setText(
+            self.locale().toCurrencyString(
+                amount.value(total_balance),
+                amount.symbol(),
+            )
+        )
+
     def on_add_category_button_clicked(self):
         """
         Triggered when user click on addCategoryButton
@@ -231,6 +373,17 @@ class AccountTreeWidget(QWidget, Ui_AccountTreeWidget):
         """
         self.add_category(self.treeView.rootIndex())
 
+    def on_refresh_button_clicked(self):
+        """
+        Triggered when user click on Refresh button
+
+        :return:
+        """
+        self.refreshButton.setEnabled(False)
+        self.errorLabel.setText("")
+        self.error_message = None
+        self.fetch_from_network_async_qworker.start()
+
     def on_category_add_event(self, _):
         """
         When add category event is triggered
@@ -246,6 +399,16 @@ class AccountTreeWidget(QWidget, Ui_AccountTreeWidget):
         :return:
         """
         self._update_model()
+        self.update_ui()
+
+    def on_unit_event(self, _):
+        """
+        When a unit event is triggered
+
+        :return:
+        """
+        self._update_model()
+        self.update_ui()
 
     def on_currency_event(self, _):
         """
@@ -254,6 +417,7 @@ class AccountTreeWidget(QWidget, Ui_AccountTreeWidget):
         :return:
         """
         self._update_model()
+        self.update_ui()
 
     def on_add_account_event(self, _):
         """
@@ -262,6 +426,7 @@ class AccountTreeWidget(QWidget, Ui_AccountTreeWidget):
         :return:
         """
         self._update_model()
+        self.update_ui()
 
     def on_delete_account_event(self, _):
         """
@@ -270,6 +435,7 @@ class AccountTreeWidget(QWidget, Ui_AccountTreeWidget):
         :return:
         """
         self._update_model()
+        self.update_ui()
 
     def on_update_account_event(self, _):
         """
@@ -278,6 +444,7 @@ class AccountTreeWidget(QWidget, Ui_AccountTreeWidget):
         :return:
         """
         self._update_model()
+        self.update_ui()
 
     def on_context_menu(self, position: QPoint):
         """
@@ -299,6 +466,47 @@ class AccountTreeWidget(QWidget, Ui_AccountTreeWidget):
                 self.application, current_element, self.mutex, self
             ).exec_(self.treeView.mapToGlobal(position))
 
+    def fetch_from_network(self):
+        """
+        Fetch table model accounts data from the network
+
+        :return:
+        """
+        accounts = self.application.accounts.get_list()
+        try:
+            self.application.accounts.network_update_balances(
+                self.application.accounts.get_list()
+            )
+        except Exception as exception:
+            logging.exception(exception)
+            self.error_message = str(exception)
+
+        addresses = [account.address for account in accounts]
+        try:
+            self.application.identities.network_update_identities(addresses)
+        except Exception as exception:
+            logging.exception(exception)
+            self.error_message = str(exception)
+        else:
+            identity_indice = self.application.identities.list_indice()
+            try:
+                self.application.smiths.network_update_smiths(identity_indice)
+            except Exception as exception:
+                logging.exception(exception)
+                self.error_message = str(exception)
+
+    def _on_finished_fetch_from_network(self):
+        """
+        Triggered when async request fetch_from_network is finished
+
+        :return:
+        """
+        logging.debug("Account tree widget update")
+
+        self.refreshButton.setEnabled(True)
+        self._update_model()
+        self.update_ui()
+
 
 if __name__ == "__main__":
     qapp = QApplication(sys.argv)
@@ -306,7 +514,7 @@ if __name__ == "__main__":
     application_ = Application(DATA_PATH)
 
     main_window = QMainWindow()
-    main_window.show()
+    main_window.showMaximized()
 
     main_window.setCentralWidget(AccountTreeWidget(application_, QMutex(), main_window))
 
