@@ -17,15 +17,16 @@ import sys
 from collections import OrderedDict
 from typing import Optional
 
-from PyQt5.QtCore import QTimer
-from PyQt5.QtGui import QFont
+from mnemonic import Mnemonic
+from mnemonic.mnemonic import ConfigurationError
+from PyQt5.QtCore import Qt, QTimer
+from PyQt5.QtGui import QFont, QKeyEvent
 from PyQt5.QtWidgets import QApplication, QDialog, QLineEdit, QWidget
-from substrateinterface import Keypair, KeypairType
 
 from tikka.domains.application import Application
-from tikka.domains.entities.account import Account
+from tikka.domains.entities.account import Account, AccountCryptoType
 from tikka.domains.entities.constants import DATA_PATH
-from tikka.libs.derivation import MEMBER_DERIVATION_PATH
+from tikka.libs.keypair import Keypair, KeypairType
 from tikka.libs.secret import sanitize_mnemonic_string
 from tikka.slots.pyqt.entities.constants import (
     ADDRESS_MONOSPACE_FONT_NAME,
@@ -41,6 +42,17 @@ class AccountDerivationCreateWindow(QDialog, Ui_AccountDerivationCreateDialog):
     AccountDerivationCreateWindow class
     """
 
+    language_code_map = {
+        "english": "en",
+        "french": "fr",
+        "chinese_simplified": "zh-hans",
+        "chinese_traditional": "zh-hant",
+        "italian": "it",
+        "japanese": "ja",
+        "korean": "ko",
+        "spanish": "es",
+    }
+
     def __init__(
         self,
         application: Application,
@@ -64,7 +76,7 @@ class AccountDerivationCreateWindow(QDialog, Ui_AccountDerivationCreateDialog):
         self.password = None
 
         # Mnemonic language selector translated
-        mnemonic_language_selector = OrderedDict(
+        self.mnemonic_language_displayed = OrderedDict(
             [
                 ("en", self._("English")),
                 ("fr", self._("French")),
@@ -76,26 +88,12 @@ class AccountDerivationCreateWindow(QDialog, Ui_AccountDerivationCreateDialog):
                 ("es", self._("Spanish")),
             ]
         )
-        for language_code, language_name in mnemonic_language_selector.items():
-            self.mnemonicLanguageComboBox.addItem(language_name, userData=language_code)
-        self.mnemonicLanguageComboBox.setCurrentIndex(
-            self.mnemonicLanguageComboBox.findData(
-                self.application.config.get("language")[:2]
-            )
-        )
 
-        # Derivation selector translated
+        # derivation selector
         for derivation_path in self.application.accounts.get_available_derivation_list(
             self.account
         ):
-            if derivation_path == MEMBER_DERIVATION_PATH:
-                self.derivationComboBox.addItem(
-                    self._("Member"), userData=MEMBER_DERIVATION_PATH
-                )
-            else:
-                self.derivationComboBox.addItem(
-                    derivation_path, userData=derivation_path
-                )
+            self.derivationComboBox.addItem(derivation_path, userData=derivation_path)
         monospace_font = QFont(ADDRESS_MONOSPACE_FONT_NAME)
         monospace_font.setStyleHint(QFont.Monospace)
         self.addressValueLabel.setFont(monospace_font)
@@ -107,8 +105,8 @@ class AccountDerivationCreateWindow(QDialog, Ui_AccountDerivationCreateDialog):
         # events
         self.mnemonicLineEdit.textChanged.connect(self._on_mnemonic_changed)
         self.showButton.clicked.connect(self.on_show_button_clicked)
-        self.mnemonicLanguageComboBox.currentIndexChanged.connect(
-            self._on_mnemonic_changed
+        self.derivationComboBox.lineEdit().keyPressEvent = (
+            self._on_derivation_keypress_event
         )
         self.derivationComboBox.currentIndexChanged.connect(self._generate_address)
         self.buttonBox.accepted.connect(self.on_accepted_button)
@@ -121,6 +119,18 @@ class AccountDerivationCreateWindow(QDialog, Ui_AccountDerivationCreateDialog):
         # fill form
         self.rootAddressValueLabel.setText(self.account.address)
 
+    def _on_derivation_keypress_event(self, event: QKeyEvent):
+        """
+
+        :param event:
+        :return:
+        """
+        if event.key() == Qt.Key_Return:
+            return
+        # if the key is not return, handle normally
+        QLineEdit.keyPressEvent(self.derivationComboBox.lineEdit(), event)
+        self._generate_address()
+
     def _on_mnemonic_changed(self):
         """
         Triggered when mnemonic is changed
@@ -150,11 +160,26 @@ class AccountDerivationCreateWindow(QDialog, Ui_AccountDerivationCreateDialog):
             self.mnemonic_debounce_timer.stop()
 
         self.errorLabel.setText("")
-        language_code = self.mnemonicLanguageComboBox.currentData()
         mnemonic = sanitize_mnemonic_string(self.mnemonicLineEdit.text())
-        derivation_path = self.derivationComboBox.currentData()
+        try:
+            self.language_code = self.language_code_map[
+                Mnemonic.detect_language(mnemonic)
+            ]
+        except ConfigurationError as exception:
+            logging.exception(exception)
+            self.addressValueLabel.setText("")
+            self.errorLabel.setText(self._("Language not detected"))
+            self.mnemonicLanguageValueLabel.setText("")
+            self.buttonBox.button(self.buttonBox.Ok).setEnabled(False)
+            return False
+
+        self.mnemonicLanguageValueLabel.setText(
+            self.mnemonic_language_displayed[self.language_code]
+        )
+
+        derivation_path = self.derivationComboBox.currentText()
         suri = mnemonic + derivation_path
-        if not Keypair.validate_mnemonic(mnemonic, language_code):
+        if not Keypair.validate_mnemonic(mnemonic, self.language_code):
             self.errorLabel.setText(self._("Mnemonic or language not valid!"))
             self.buttonBox.button(self.buttonBox.Ok).setEnabled(False)
             return False
@@ -163,8 +188,8 @@ class AccountDerivationCreateWindow(QDialog, Ui_AccountDerivationCreateDialog):
             root_keypair = Keypair.create_from_mnemonic(
                 mnemonic=mnemonic,
                 ss58_format=self.application.currencies.get_current().ss58_format,
-                crypto_type=KeypairType.SR25519,
-                language_code=language_code,
+                crypto_type=self.account.crypto_type,
+                language_code=self.language_code,
             )
         except Exception as exception:
             logging.exception(exception)
@@ -181,11 +206,11 @@ class AccountDerivationCreateWindow(QDialog, Ui_AccountDerivationCreateDialog):
             return False
 
         try:
-            self.keypair = Keypair.create_from_uri(
+            self.keypair = Keypair.create_from_uri(  # type: ignore
                 suri=suri,
                 ss58_format=self.application.currencies.get_current().ss58_format,
-                crypto_type=KeypairType.SR25519,
-                language_code=language_code,
+                crypto_type=self.account.crypto_type or KeypairType.ED25519,
+                language_code=self.language_code,
             )
         except Exception as exception:
             logging.exception(exception)
@@ -194,11 +219,20 @@ class AccountDerivationCreateWindow(QDialog, Ui_AccountDerivationCreateDialog):
             return False
 
         if self.keypair is not None:
+            self.addressValueLabel.setText(self.keypair.ss58_address)
+
+            if (
+                self.application.accounts.get_by_address(self.keypair.ss58_address)
+                is not None
+            ):
+                self.errorLabel.setText(self._("Account already exists!"))
+                self.buttonBox.button(self.buttonBox.Ok).setEnabled(False)
+                return False
+
             stored_password = self.application.passwords.get_clear_password(
                 root_keypair
             )
             self.storedpasswordLineEdit.setText(stored_password)
-            self.addressValueLabel.setText(self.keypair.ss58_address)
             self.buttonBox.button(self.buttonBox.Ok).setEnabled(True)
 
         return True
@@ -226,15 +260,14 @@ class AccountDerivationCreateWindow(QDialog, Ui_AccountDerivationCreateDialog):
         """
         # user inputs
         mnemonic = sanitize_mnemonic_string(self.mnemonicLineEdit.text())
-        derivation_path = self.derivationComboBox.currentData()
-        language_code = self.mnemonicLanguageComboBox.currentData()
+        derivation_path = self.derivationComboBox.currentText()
         name = self.nameLineEdit.text().strip()
 
         # generated inputs
         root_keypair = Keypair.create_from_mnemonic(
             mnemonic=mnemonic,
-            language_code=language_code,
-            crypto_type=KeypairType.SR25519,
+            language_code=self.language_code,
+            crypto_type=self.account.crypto_type,
             ss58_format=self.application.currencies.get_current().ss58_format,
         )
         password = self.application.passwords.get_clear_password(root_keypair)
@@ -242,14 +275,24 @@ class AccountDerivationCreateWindow(QDialog, Ui_AccountDerivationCreateDialog):
 
         # create derived account + wallet (and the read-only root account if not exists)
         self.application.accounts.create_new_account(
-            mnemonic, language_code, derivation_path, name, password
+            mnemonic,
+            self.language_code,
+            derivation_path,
+            self.account.crypto_type,
+            name,
+            password,
         )
 
 
 if __name__ == "__main__":
     qapp = QApplication(sys.argv)
     application_ = Application(DATA_PATH)
-    account_ = Account(
-        "5E6taVoL7eW8qpPKjDcf3GjyvFkkMB8EHkkqZksiWeViEkSP", name="test name"
+    account_ = application_.accounts.create_new_root_account(
+        "album cute glance oppose hub fury strategy health dust rebuild trophy magic",
+        "en",
+        AccountCryptoType.ED25519,
+        "test root account",
+        "aaaaaa",
     )
+    assert account_ is not None
     AccountDerivationCreateWindow(application_, account_).exec_()
