@@ -12,22 +12,39 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
+import logging
 import sys
 from collections import OrderedDict
 from typing import Optional
 
-from PyQt5.QtCore import QTimer
-from PyQt5.QtGui import QFont
-from PyQt5.QtWidgets import QApplication, QDialog, QLineEdit, QWidget
-from substrateinterface import Keypair, KeypairType
+from mnemonic import Mnemonic
+from mnemonic.mnemonic import ConfigurationError
+from PyQt5.QtCore import QMutex, Qt, QTimer
+from PyQt5.QtGui import QFont, QMovie, QPixmap
+from PyQt5.QtWidgets import QApplication, QButtonGroup, QDialog, QLineEdit, QWidget
 
 from tikka.domains.application import Application
-from tikka.domains.entities.constants import DATA_PATH, WALLETS_PASSWORD_LENGTH
+from tikka.domains.entities.account import AccountCryptoType
+from tikka.domains.entities.constants import (
+    AMOUNT_UNIT_KEY,
+    DATA_PATH,
+    WALLETS_PASSWORD_LENGTH,
+)
+from tikka.domains.entities.identity import Identity, IdentityStatus
+from tikka.domains.entities.smith import Smith, SmithStatus
+from tikka.libs.keypair import Keypair
 from tikka.libs.secret import generate_alphabetic, sanitize_mnemonic_string
 from tikka.slots.pyqt.entities.constants import (
     ADDRESS_MONOSPACE_FONT_NAME,
     DEBOUNCE_TIME,
+    ICON_IDENTITY_MEMBER,
+    ICON_IDENTITY_MEMBER_NOT_VALIDATED,
+    ICON_IDENTITY_NOT_MEMBER,
+    ICON_LOADER,
+    ICON_SMITH,
+    SELECTED_UNIT_PREFERENCES_KEY,
 )
+from tikka.slots.pyqt.entities.worker import AsyncQWorker
 from tikka.slots.pyqt.resources.gui.windows.account_import_rc import (
     Ui_AccountImportDialog,
 )
@@ -38,11 +55,25 @@ class AccountImportWindow(QDialog, Ui_AccountImportDialog):
     AccountImportWindow class
     """
 
-    def __init__(self, application: Application, parent: Optional[QWidget] = None):
+    language_code_map = {
+        "english": "en",
+        "french": "fr",
+        "chinese_simplified": "zh-hans",
+        "chinese_traditional": "zh-hant",
+        "italian": "it",
+        "japanese": "ja",
+        "korean": "ko",
+        "spanish": "es",
+    }
+
+    def __init__(
+        self, application: Application, mutex: QMutex, parent: Optional[QWidget] = None
+    ):
         """
         Init import account window
 
         :param application: Application instance
+        :param mutex: QMutex instance
         :param parent: QWidget instance
         """
         super().__init__(parent=parent)
@@ -50,9 +81,11 @@ class AccountImportWindow(QDialog, Ui_AccountImportDialog):
 
         self.application = application
         self._ = self.application.translator.gettext
+        self.language_code = "en"
+        self.mutex = mutex
 
         # Mnemonic language selector translated
-        mnemonic_language_selector = OrderedDict(
+        self.mnemonic_language_displayed = OrderedDict(
             [
                 ("en", self._("English")),
                 ("fr", self._("French")),
@@ -64,27 +97,63 @@ class AccountImportWindow(QDialog, Ui_AccountImportDialog):
                 ("es", self._("Spanish")),
             ]
         )
-        for language_code, language_name in mnemonic_language_selector.items():
-            self.mnemonicLanguageComboBox.addItem(language_name, userData=language_code)
-        self.mnemonicLanguageComboBox.setCurrentIndex(
-            self.mnemonicLanguageComboBox.findData(
-                self.application.config.get("language")[:2]
-            )
+
+        self.balance: Optional[int] = None
+        self.identity: Optional[Identity] = None
+        self.smith: Optional[Smith] = None
+
+        self.display_identity_icon = {
+            IdentityStatus.UNCONFIRMED.value: QPixmap(
+                ICON_IDENTITY_MEMBER_NOT_VALIDATED
+            ),
+            IdentityStatus.UNVALIDATED.value: QPixmap(
+                ICON_IDENTITY_MEMBER_NOT_VALIDATED
+            ),
+            IdentityStatus.MEMBER.value: QPixmap(ICON_IDENTITY_MEMBER),
+            IdentityStatus.NOT_MEMBER.value: QPixmap(ICON_IDENTITY_NOT_MEMBER),
+            IdentityStatus.REVOKED.value: QPixmap(ICON_IDENTITY_NOT_MEMBER),
+        }
+
+        # calculate balance display by unit preference
+        unit_preference = self.application.repository.preferences.get(
+            SELECTED_UNIT_PREFERENCES_KEY
         )
+        if unit_preference is not None:
+            self.amount = self.application.amounts.get_amount(unit_preference)
+        else:
+            self.amount = self.application.amounts.get_amount(AMOUNT_UNIT_KEY)
+
         monospace_font = QFont(ADDRESS_MONOSPACE_FONT_NAME)
         monospace_font.setStyleHint(QFont.Monospace)
         self.addressValueLabel.setFont(monospace_font)
 
+        # animated loading icon
+        self.loader_movie = QMovie(ICON_LOADER)
+        self.loader_movie.start()
+        self.loaderIconLabel.setMovie(self.loader_movie)
+        loader_icon_size_policy = self.loaderIconLabel.sizePolicy()
+        loader_icon_size_policy.setRetainSizeWhenHidden(True)
+        self.loaderIconLabel.setSizePolicy(loader_icon_size_policy)
+        self.loaderIconLabel.hide()
+
         # buttons
         self.buttonBox.button(self.buttonBox.Ok).setEnabled(False)
 
+        self.cryptoTypeGroup = QButtonGroup(self)
+        self.cryptoTypeGroup.addButton(self.ed25519RadioButton)
+        self.cryptoTypeGroup.addButton(self.sr25519RadioButton)
+        self.ed25519RadioButton.setChecked(True)
+        self.crypto_type = (
+            AccountCryptoType.ED25519
+            if self.ed25519RadioButton.isChecked()
+            else AccountCryptoType.SR25519
+        )
+
         # events
         self.mnemonicLineEdit.textChanged.connect(self._on_mnemonic_changed)
         self.showButton.clicked.connect(self.on_show_button_clicked)
-        self.mnemonicLanguageComboBox.currentIndexChanged.connect(
-            self._on_mnemonic_changed
-        )
         self.derivationLineEdit.textChanged.connect(self._on_derivation_changed)
+        self.cryptoTypeGroup.buttonClicked.connect(self.on_crypto_type_changed)
         self.passwordChangeButton.clicked.connect(self._generate_wallet_password)
         self.buttonBox.accepted.connect(self.on_accepted_button)
         self.buttonBox.rejected.connect(self.close)
@@ -95,6 +164,14 @@ class AccountImportWindow(QDialog, Ui_AccountImportDialog):
         self.derivation_debounce_timer = QTimer()
         self.derivation_debounce_timer.timeout.connect(self._derivation_debounce_call)
 
+        # Create a QWorker object
+        self.network_fetch_account_data_async_qworker = AsyncQWorker(
+            self.network_fetch_account_data, self.mutex
+        )
+        self.network_fetch_account_data_async_qworker.finished.connect(
+            self._on_finished_network_fetch_account_data
+        )
+
         # fill form
         self.storedPasswordFrame.hide()
         self._generate_wallet_password()
@@ -134,15 +211,11 @@ class AccountImportWindow(QDialog, Ui_AccountImportDialog):
         if self.address_already_exists(address):
             return
 
-        derivation = self.derivationLineEdit.text().strip()
-        if derivation == "":
-            self.errorLabel.setText(
-                self._("WARNING : it is not secure to store a root account private key")
-            )
-
         self._update_wallet_password_fields()
 
-        self.buttonBox.button(self.buttonBox.Ok).setEnabled(True)
+        if self.application.connections.node.is_connected():
+            self.loaderIconLabel.show()
+            self.network_fetch_account_data_async_qworker.start()
 
     def on_show_button_clicked(self):
         """
@@ -180,7 +253,6 @@ class AccountImportWindow(QDialog, Ui_AccountImportDialog):
             self.derivation_debounce_timer.stop()
 
         self.errorLabel.setText("")
-        derivation = self.derivationLineEdit.text().strip()
 
         try:
             address = self._generate_address()
@@ -189,17 +261,14 @@ class AccountImportWindow(QDialog, Ui_AccountImportDialog):
             self.buttonBox.button(self.buttonBox.Ok).setEnabled(False)
             return
 
-        if derivation == "":
-            self.errorLabel.setText(
-                self._("WARNING : it is not secure to store a root account private key")
-            )
-
         if self.address_already_exists(address):
             return
 
         self._update_wallet_password_fields()
 
-        self.buttonBox.button(self.buttonBox.Ok).setEnabled(True)
+        if self.application.connections.node.is_connected():
+            self.loaderIconLabel.show()
+            self.network_fetch_account_data_async_qworker.start()
 
     def _update_wallet_password_fields(self):
         """
@@ -209,11 +278,10 @@ class AccountImportWindow(QDialog, Ui_AccountImportDialog):
         """
         # if root account exists, hide the password field
         mnemonic = sanitize_mnemonic_string(self.mnemonicLineEdit.text())
-        language_code = self.mnemonicLanguageComboBox.currentData()
         root_keypair = Keypair.create_from_mnemonic(
             mnemonic=mnemonic,
-            language_code=language_code,
-            crypto_type=KeypairType.SR25519,
+            language_code=self.language_code,
+            crypto_type=self.crypto_type,
             ss58_format=self.application.currencies.get_current().ss58_format,
         )
         if self.application.passwords.exists(root_keypair.ss58_address):
@@ -227,6 +295,23 @@ class AccountImportWindow(QDialog, Ui_AccountImportDialog):
             self.storedPasswordFrame.hide()
             self.passwordFrame.show()
 
+    def on_crypto_type_changed(self):
+        """
+        Triggered when user click on key type radio button
+
+        :return:
+        """
+        self.errorLabel.setText("")
+        self.crypto_type = (
+            AccountCryptoType.ED25519
+            if self.ed25519RadioButton.isChecked()
+            else AccountCryptoType.SR25519
+        )
+        if self.mnemonicLineEdit.text().strip() != "":
+            address = self._generate_address()
+            if not self.address_already_exists(address):
+                self.buttonBox.button(self.buttonBox.Ok).setEnabled(True)
+
     def _generate_address(self) -> str:
         """
         Generate, display and return address
@@ -235,13 +320,13 @@ class AccountImportWindow(QDialog, Ui_AccountImportDialog):
         """
         # sanitize mnemonic string
         mnemonic = sanitize_mnemonic_string(self.mnemonicLineEdit.text())
-        language_code = self.mnemonicLanguageComboBox.currentData()
+
         derivation = self.derivationLineEdit.text().strip()
         address = Keypair.create_from_uri(
             suri=mnemonic + derivation,
             ss58_format=self.application.currencies.get_current().ss58_format,
-            crypto_type=KeypairType.SR25519,
-            language_code=language_code,
+            crypto_type=self.crypto_type,
+            language_code=self.language_code,
         ).ss58_address
 
         self.addressValueLabel.setText(address)
@@ -269,12 +354,29 @@ class AccountImportWindow(QDialog, Ui_AccountImportDialog):
         """
         # verify mnemonic
         mnemonic = sanitize_mnemonic_string(self.mnemonicLineEdit.text())
-        language_code = self.mnemonicLanguageComboBox.currentData()
-        if not Keypair.validate_mnemonic(mnemonic, language_code):
-            self.errorLabel.setText(self._("Mnemonic or language not valid!"))
+        try:
+            self.language_code = self.language_code_map[
+                Mnemonic.detect_language(mnemonic)
+            ]
+        except ConfigurationError as exception:
+            logging.exception(exception)
+            self.addressValueLabel.setText("")
+            self.errorLabel.setText(self._("Language not detected"))
+            self.mnemonicLanguageValueLabel.setText("")
             self.buttonBox.button(self.buttonBox.Ok).setEnabled(False)
             return False
 
+        if not Keypair.validate_mnemonic(mnemonic, self.language_code):
+            self.addressValueLabel.setText("")
+            self.errorLabel.setText(self._("Mnemonic not valid!"))
+            self.mnemonicLanguageValueLabel.setText("")
+            self.buttonBox.button(self.buttonBox.Ok).setEnabled(False)
+            return False
+
+        self.mnemonicLanguageValueLabel.setText(
+            self.mnemonic_language_displayed[self.language_code]
+        )
+
         return True
 
     def _generate_wallet_password(self):
@@ -285,6 +387,79 @@ class AccountImportWindow(QDialog, Ui_AccountImportDialog):
         """
         self.passwordLineEdit.setText(generate_alphabetic(WALLETS_PASSWORD_LENGTH))
 
+    def network_fetch_account_data(self):
+        """
+        Fetch account data and
+        check if account is really a known legacy v1 account
+
+        :return:
+        """
+        address = self.addressValueLabel.text().strip()
+        self.balance = None
+        self.identity = None
+        self.smith = None
+
+        try:
+            self.balance = self.application.accounts.network_get_balance(address)
+        except Exception as exception:
+            self.errorLabel.setText(self._(str(exception)))
+            logging.exception(exception)
+        else:
+            try:
+                self.identity = self.application.identities.network_get_identity(
+                    address
+                )
+            except Exception as exception:
+                self.errorLabel.setText(self._(str(exception)))
+                logging.exception(exception)
+            else:
+                if self.identity is not None:
+                    try:
+                        self.smith = self.application.smiths.network_get_smith(
+                            self.identity.index
+                        )
+                    except Exception as exception:
+                        self.errorLabel.setText(self._(str(exception)))
+                        logging.exception(exception)
+
+    def _on_finished_network_fetch_account_data(self):
+        """
+        Triggered when async request network_fetch_account_data_async_qworker is finished
+
+        :return:
+        """
+        self.loaderIconLabel.hide()
+
+        if self.balance is not None:
+            self.balanceValueLabel.setText(
+                self.locale().toCurrencyString(
+                    self.amount.value(self.balance), self.amount.symbol()
+                )
+            )
+        else:
+            self.balanceValueLabel.setText("?")
+        if self.identity:
+            self.identityIconLabel.setPixmap(
+                self.display_identity_icon[self.identity.status.value].scaled(
+                    16, 18, aspectRatioMode=Qt.KeepAspectRatio
+                )
+            )
+            identity_name = self.identity.name or ""
+            self.identityNameAndIndexValuelabel.setText(
+                f"{identity_name}#{self.identity.index}"
+            )
+            if self.smith is not None and self.smith.status == SmithStatus.SMITH:
+                self.identityIconLabel.setPixmap(
+                    QPixmap(ICON_SMITH).scaled(
+                        16, 18, aspectRatioMode=Qt.KeepAspectRatio
+                    )
+                )
+        else:
+            self.identityNameAndIndexValuelabel.setText("")
+            self.identityIconLabel.clear()
+
+        self.buttonBox.button(self.buttonBox.Ok).setEnabled(True)
+
     def on_accepted_button(self):
         """
         Triggered when user click on ok button
@@ -293,19 +468,32 @@ class AccountImportWindow(QDialog, Ui_AccountImportDialog):
         """
         # user inputs
         mnemonic = sanitize_mnemonic_string(self.mnemonicLineEdit.text())
+        derivation_ = self.derivationLineEdit.text().strip()
         name = self.nameLineEdit.text().strip()
-        language_code = self.mnemonicLanguageComboBox.currentData()
 
         # generated inputs
         password = self.passwordLineEdit.text()
-        derivation_ = self.derivationLineEdit.text().strip()
 
-        self.application.accounts.create_new_account(
-            mnemonic, language_code, derivation_, name, password
+        account = self.application.accounts.create_new_account(
+            mnemonic, self.language_code, derivation_, self.crypto_type, name, password
         )
+        if account:
+            if self.balance:
+                account.balance = self.balance
+                self.application.accounts.update(account)
+            if self.identity:
+                if self.application.identities.get(self.identity.index) is None:
+                    self.application.identities.add(self.identity)
+                else:
+                    self.application.identities.update(self.identity)
+                if self.smith:
+                    if self.application.smiths.get(self.identity.index) is None:
+                        self.application.smiths.add(self.smith)
+                    else:
+                        self.application.smiths.update(self.smith)
 
 
 if __name__ == "__main__":
     qapp = QApplication(sys.argv)
     application_ = Application(DATA_PATH)
-    AccountImportWindow(application_).exec_()
+    AccountImportWindow(application_, QMutex()).exec_()
