@@ -69,6 +69,8 @@ class AccountRenameWindow(QDialog, Ui_AccountRenameDialog):
         :return:
         """
         self.account.name = self.nameLineEdit.text().strip()
+        if self.account.name == "":
+            self.account.name = None
         self.application.accounts.update(self.account)
 
 
